`binsacc` <-
function(sacl,sacr)
{
# -------------------------------------------------------------------
# 
#   FUNCTION readbinsacc.r
#   (based on "binsacc.m", Version 2.0, 18 JUL 05)
#   Determination of binocular (micro)saccades;
#   Please cite: Engbert, R., & Mergenthaler, K. (2006) Microsaccades 
#   are triggered by low retinal image slip. Proceedings of the National 
#   Academy of Sciences of the United States of America, 103: 7192-7197.
# 
#   (matlab Version 1.0, 07 JAN 08)
#   ported to R by Jochen Laubrock
# -------------------------------------------------------------------
# 
#   INPUT: saccade matrices from FUNCTION microsacc.m
# 
#   sacl(:,1:7)       microsaccades detected from left eye
#   sacr(:,1:7)       microsaccades detected from right eye
# 
#   OUTPUT:
# 
#   sac(:,1:14)       binocular microsaccades (right eye/left eye)
#   monol(:,1:7)      monocular microsaccades of the left eye
#   monor(:,1:7)      monocular microsaccades of the right eye
# 
# ---------------------------------------------------------------------

if (nrow(sacr) * nrow(sacl) > 0) {

    # determine saccade clusters
    TR <- max(sacr[,2])
    TL <- max(sacl[,2])
    T <- max(TL, TR)
    s <- rep(0, T+1)
    for (i in 1:nrow(sacl)) {
        s[(sacl[i,1]+1):sacl[i,2]] <- 1
	}
    for (i in 1:nrow(sacr)) {
		s[(sacr[i,1]+1):sacr[i,2]] <- 1
	}
    s[1] <- 0
    s[T+1] <- 0

	
    m <- which(diff(s)!=0)
    N <- length(m)/2
    m <- cbind(m[seq(1,length(m), by=2)], m[seq(2,length(m), by=2)])

    # combine clusters separated by fixations <=5 samples
    idx <- 1
    while (idx < nrow(m)){
        if (m[idx+1,1] - m[idx,2] <= 5) {
            m[idx,2] = m[idx+1,2]	# set saccade t1 to next sacade's t1
            m <- m[-(idx+1),] 		# remove next row
		} else {
            idx = idx + 1
		}
	}
    if (m[1,1] <= 5) {
        m[1,1] <- 1
    }
    N <- nrow(m)

    # determine binocular saccades
    NB <- 0
    NR <- 0
    NL <- 0
	sac <- data.frame(matrix(nrow=N, ncol=14))  # pre-allocate N rows, empty rows will be deleted later
	monol <- data.frame(matrix(nrow=0, ncol=7)) # start empty because most MS are binocular
	monor <- data.frame(matrix(nrow=0, ncol=7))
	names(monol) <- c('t0', 't1', 'vpeak', 'dx', 'dy', 'dX', 'dY')
	names(monor) <- c('t0', 't1', 'vpeak', 'dx', 'dy', 'dX', 'dY')
	names(sac) <- c('t0_L', 't1_L', 'vpeak_L', 'dx_L', 'dy_L', 'dX_L', 'dY_L', 't0_R', 't1_R', 'vpeak_R', 'dx_R', 'dy_R', 'dX_R', 'dY_R')
    # loop over all clusters i
    for (i in 1:N) {
        r <- which( m[i,1] <= sacr[,1] & sacr[,2] <= m[i,2] )
        l <- which( m[i,1] <= sacl[,1] & sacl[,2] <= m[i,2] )
		# discuss: shouldn't the stats also be combined or at least be available for clusters?
		# the current approach always seems to use measures from the first saccade of a cluster?

        if (length(l)*length(r) > 0) {
            ampr <- sqrt(sacr[r,6]^2 + sacr[r,7]^2)
            ampl <- sqrt(sacl[l,6]^2 + sacl[l,7]^2)
            ir <- which.max(ampr);
            il <- which.max(ampl);
            ronset <- m[i,1]
            roffset <- m[i,2]
            lonset <- m[i,1]
            loffset <- m[i,2]
            #ronset = sacr(r(1),1);
            #roffset = sacr(r(end),2);
            #lonset = sacl(l(1),1);
            #loffset = sacl(l(end),2);
            NB <- NB + 1
            sac[NB,] <- as.vector(unlist(c(ronset, roffset, sacr[r[ir],3:7], lonset, loffset, sacl[l[il],3:7])))
            #  sac[1:num,1]   onset of saccade
            #  sac[1:num,2]   end of saccade
            #  sac[1:num,3]   peak velocity of saccade (vpeak)
            #  sac[1:num,4]   horizontal component     (dx)
            #  sac[1:num,5]   vertical component       (dy)
            #  sac[1:num,6]   horizontal amplitude     (dX)
            #  sac[1:num,7]   vertical amplitude       (dY)
		} else {
            # determine monocular saccades
			#             for (j in 1:length(r)) {
			#                 NR <- NR + 1
			#                 monor <- rbind(monor, sacr[r[j],])
			#             }
			#             for (j in 1:length(l)) {
			#                 NL <- NL + 1
			#                 monol <- rbind(monol, sacl[l[j],])
			# }
			if (length(l)==0) {
	            NR <- NR + length(r)
	            monor <- rbind(monor, sacr[r,])
			}
			if (length(r)==0) {
	            NL <- NL + length(l)
	            monol <- rbind(monol, sacl[l,])}
		}
	}
	# remove empty rows
	if (NB < N) {
		 sac <- sac[-((NB+1):N),] 
	} 
} else {
    # special cases of exclusively monocular saccades (saccades or trials? jl)
    if (nrow(sacr)==0) {
        sac <- NA
        monor <- NA
        monol <- sacl
    }
    if (nrow(sacl)==0) {
        sac <- NA
        monol <- NA
        monor <- sacr;
    }
}
monol <- data.frame(monol)
monor <- data.frame(monor)
sac <- data.frame(sac)
names(monol) <- c('t0', 't1', 'vpeak', 'dx', 'dy', 'dX', 'dY')
names(monor) <- c('t0', 't1', 'vpeak', 'dx', 'dy', 'dX', 'dY')
names(sac) <- c('t0_L', 't1_L', 'vpeak_L', 'dx_L', 'dy_L', 'dX_L', 'dY_L', 't0_R', 't1_R', 'vpeak_R', 'dx_R', 'dy_R', 'dX_R', 'dY_R')
return(list(sac=sac, monol=monol, monor=monor))
}

