`microsacc` <- function(x,vel,VFAC=5, MINDUR=3, discardMS=TRUE, removeGlissades=TRUE, DTGLISS=30, glissadesBelongToFixation=FALSE, MSamplCrit=0.76)
{
#-------------------------------------------------------------------
# based on matlab
#  FUNCTION microsacc.m
#
#  (Version 1.0, 22 FEB 01)
#  (Version 2.0, 18 JUL 05)
#  (Version 2.1, 03 OCT 05)
#
# R port by jochen laubrock 09
#-------------------------------------------------------------------
#
#  INPUT:
#
#  x(:,1:2)         position vector
#  vel(:,1:2)       velocity vector
#  VFAC             relative velocity threshold
#  MINDUR           minimal saccade duration
#  removeGlissades  whether to remove glissades (overshoot correction)
#  DTGLISS          minimum time between two saccades, if shorter, sacc is considered glissade
#  glissadesBelongToFixation whether to add glissade durations to fixation; otherwise they are added to saccade
#  discardMS        logical value indicating whether to keep or discard microsaccades (default: TRUE); 
#  MSamplCrit       minimum saccade amplitude (in degrees) if discardMS==TRUE
#
# note: microsaccade removal is currently very poorly implemented, based on an absolute saccade amplitude criterion
#
#  OUTPUT:
#
#  sac(1:num,1)   onset of saccade
#  sac(1:num,2)   end of saccade
#  sac(1:num,3)   peak velocity of saccade (vpeak)
#  sac(1:num,4)   horizontal component     (dx)
#  sac(1:num,5)   vertical component       (dy)
#  sac(1:num,6)   horizontal amplitude     (dX)
#  sac(1:num,7)   vertical amplitude       (dY)
#
#---------------------------------------------------------------------

# compute threshold
medx <- median(vel[,1])
msdx <- sqrt( median((vel[,1]-medx)^2, na.rm=TRUE) )
medy <- median(vel[,2])
msdy <- sqrt( median((vel[,2]-medy)^2, na.rm=TRUE) )
#msdx = sqrt( median(vel(:,1).^2) - (median(vel(:,1)))^2 );
#msdy = sqrt( median(vel(:,2).^2) - (median(vel(:,2)))^2 );
realmin <- .Machine$double.eps
if (msdx<realmin){
    msdx <- sqrt( mean(vel[,1]^2, na.rm = TRUE) - (mean(vel[,1], na.rm = TRUE))^2 )
}
if (msdy<realmin){
	msdy <- sqrt( mean(vel[,2]^2, na.rm = TRUE) - (mean(vel[,2], na.rm = TRUE))^2 )
}
radiusx = VFAC * msdx
radiusy = VFAC * msdy
radius <- data.frame(cbind(radiusx, radiusy))
names(radius) <- c('x', 'y')

# compute test criterion: ellipse equation
test <- (vel[,1]/radiusx)^2 + (vel[,2]/radiusy)^2
indx <- which(test>1)

# determine saccades
N <- length(indx)
sac <- NULL
nsac <- 0
dur <- 1
a <- 1
k <- 1
while (k < N) {
    if ((indx[k+1] - indx[k])==1)
        dur <- dur + 1
    else{
        if (dur >= MINDUR){
            nsac <- nsac + 1
            b <- k
            sac <- rbind(sac,c(indx[a], indx[b]))
        }
        a <- k+1
        dur <- 1
    }
    k <- k + 1
}

# check for minimum duration
if (dur >= MINDUR) {
    nsac <- nsac + 1
    b <- k
    sac <- rbind(sac,c(indx[a], indx[b]))
}

sac <- cbind(sac, matrix(NA, nrow=nrow(sac), ncol=5))
# compute peak velocity, horizonal and vertical components
for (s in 1:nsac) {
    # onset and offset
    a <- sac[s,1]
    b <- sac[s,2] 
    # saccade peak velocity (vpeak)
    vpeak <- max( sqrt( vel[a:b,1]^2 + vel[a:b,2]^2 ) )
    sac[s,3] <- vpeak
    # saccade vector (dx,dy)
    dx <- x[b,1] - x[a,1]
    dy <- x[b,2] - x[a,2] 
    sac[s,4] <- dx
    sac[s,5] <- dy
    # saccade amplitude (dX,dY)
    i <- sac[s,1]:sac[s,2]
    minx <- min(x[i,1])
    maxx <- max(x[i,1])
    miny <- min(x[i,2])
    maxy <- max(x[i,2])
	ix1 <- which.min(x[i,1])
	ix2 <- which.max(x[i,1])
	iy1 <- which.min(x[i,2])
	iy2 <- which.max(x[i,2])
    dX <- sign(ix2-ix1)*(maxx-minx)
    dY <- sign(iy2-iy1)*(maxy-miny)
    sac[s,6:7] <- c(dX, dY)
}

sac <- data.frame(matrix(sac, ncol=7))
names(sac) <- c('t0', 't1', 'vpeak', 'dx', 'dy', 'dX', 'dY')

if (nsac>0 && removeGlissades){
# idea: heuristically remove glissades from detected saccades
# algorithm: take largest saccade in a neighborhood of DT samples (DT small!), adjust statistics
	sactmp <- removeGlissades(sac,  DTGLISS=DTGLISS, glissadesBelongToFixation=glissadesBelongToFixation)
	sac <- sactmp$sac 
	hasGlissade <- sactmp$hasGlissade
}

# discard microsaccades (for reading)
if (nsac>0 && discardMS ){
    dist <- sqrt( sac[,4]^2 + sac[,5]^2 )
    idx <- dist>=MSamplCrit # used by Ralf: 0.76, used by Sarah: 0.82
    sac <- sac[idx,]
}

results = list(sac=sac, radius=radius)

return(results)
}