removeGlissades <- function(sac,  DTGLISS=30, glissadesBelongToFixation=TRUE) 
# idea: heuristically remove glissades from detected saccades
# algorithm: take largest saccade in a neighborhood of DT samples (DT small!), adjust statistics
#
# jochen laubrock, 03/2010
{

	computeEnsembleStats <- function(sacensemble)
	{
		stopifnot(nrow(sacensemble)>1)

		idxs <- rep(NA, 3)
		idxs[1] <- which.max(abs(sacensemble$vpeak))
		idxs[2] <- which.max(abs(sacensemble$dX))
		idxs[3] <- which.max(abs(sacensemble$dY))

		b <- sacensemble[1,]
		# sum of components
		b$dx <- sum(sacensemble$dx)
		b$dy <- sum(sacensemble$dy)
		# maximum velocity
		b$vpeak <- sacensemble[idxs[1],]$vpeak
		# maximum of amplitudes
		b$dX <- sacensemble[idxs[2],]$dX
		b$dY <- sacensemble[idxs[3],]$dY
		# start and finishing time of fastest saccade in ensemble
		b$t0 <- sacensemble[idxs[1],]$t0
		b$t1 <- sacensemble[idxs[1],]$t1
		# start time of first and finishing time of last saccade in ensemble
		b$t0a <- sacensemble[1,]$t0
		b$t1a <- sacensemble[nrow(sacensemble),]$t1

		b
	}

	# difference between offset of sac n and onset of sac n+1
	# dt <- c(sac[1, "t0"], sac[2:(nrow(sac)),"t0"] - sac[1:(nrow(sac)-1),"t1"])
	dt <- sac[2:(nrow(sac)),"t0"] - sac[1:(nrow(sac)-1),"t1"]
	ixlGlissade <- dt <= DTGLISS
	if (!any(ixlGlissade)) 
 	{ # no glissades in trial
		newsac <- sac
		newsac$t0a <- NA
		newsac$t1a <- NA
		hasGlissade <- NA		
	} else {
		isGlissade <- which(ixlGlissade)+1 # +1 because of diff in dt
	
		#  move along potential glissades; find largest saccades of ensemble
		tg <- sac[isGlissade, 1:2]
		dt2 <- NULL
		if (nrow(tg)>=2) {dt2 <- tg[2:nrow(tg),1] - tg[1:(nrow(tg)-1),2]}
		isGlissadeSeq <- which(diff(tg[,1]) <= DTGLISS | diff(tg[,2]) <= DTGLISS)
		isGlissadeSeq <- sort(union(isGlissadeSeq, which(dt2<=DTGLISS)))

		# identify between glissade sequences longer than two, which deserve special treatment below
		isGlissadeSeq <- sort(unique(c(isGlissadeSeq, isGlissadeSeq+1)))
		isGlissadeNoSeq <- setdiff(1:nrow(tg), isGlissadeSeq)
	
		ixGlissadeSeq <- isGlissade[isGlissadeSeq]
		ixGlissadeNoSeq <- isGlissade[isGlissadeNoSeq]

		# if no glissade, copy original saccades
		toBeCorrected <- sort(union(isGlissade, isGlissade-1))
		keepOriginal <- setdiff(1:nrow(sac), toBeCorrected)
		if (length(toBeCorrected)>0 & length(toBeCorrected)<nrow(sac))  {
			newsac <- sac[keepOriginal, ]
			newsac$t0a <- NA
			newsac$t1a <- NA
		} else {
			newsac <- data.frame(sac[NULL,], t0a=numeric(0), t1a=numeric(0))
		}
		# if no glissade sequence, combine stats according to computeEnsembleStats
		for (i in seq_along(ixGlissadeNoSeq)){
			ensemble <- rbind(sac[(ixGlissadeNoSeq-1)[i],], sac[ixGlissadeNoSeq[i],])
			# for debugging
			# if (nrow(ensemble)<=1) {browser()}
			tmpsac <- computeEnsembleStats(ensemble)
			newsac <- rbind(newsac, tmpsac)
		}
	
		# if glissade sequence, combine stats according to computeEnsembleStats
		moreGlissadesIx <- which(diff(ixGlissadeSeq)>1) 
		if (length(isGlissadeSeq)>0){
			if (length(moreGlissadesIx)==0) {
				stopIx <- length(ixGlissadeSeq)
				startIx <- 1
			} else {
				stopIx <- c(moreGlissadesIx,  length(ixGlissadeSeq)) 
				startIx <- c(1,  stopIx[1:(length(stopIx)-1)]+1)
			}
			for (i in seq_along(1:length(startIx))) {
				start <- startIx[i]
				stop <- stopIx[i]
				ensemble <- sac[ixGlissadeSeq[start]-1,]
				ensemble <- rbind(ensemble, sac[ixGlissadeSeq[start:stop],])
				# for debugging
				# if (nrow(ensemble)<=1) {browser()}
				tmpsac <- computeEnsembleStats(ensemble)
				newsac <- rbind(newsac, tmpsac)
			}
		}
		newsac <- newsac[order(newsac$t0),]
		newsac
	
		sac <- newsac[, c('t0', 't1', 'vpeak', 'dx', 'dy', 'dX', 'dY')]
		hasGlissade <- rep(FALSE, nrow(newsac))
		hasGlissade[!is.na(newsac$t1a)] <- TRUE
		if(!glissadesBelongToFixation) {
			sac[hasGlissade, "t0"] <- newsac[hasGlissade, "t0a"]
			sac[hasGlissade, "t1"] <- newsac[hasGlissade, "t1a"]
		}
	} 
	list(sac=sac, hasGlissade=hasGlissade)
}
