removeShortFixationsBinoc <- function(aL, aR, MIN_DUR=14) 
{
# # remove extremely short fixations, likely to be 'overshoot-correction' artifacts
# inputs: 
#	aL, aR: fixation sequences from left and right eye, respectively
#		labeled data frame, containing as a minimum the variables "fdur", "sacdur", "x_component", "y_component", "xampl", "yampl", "peakvel"
# 	MIN_DUR: minimum duration of a fixation to keep
# output:
#	2-element list containing fixation sequences for left and right eye, respectively, with short fixations removed
# 		an extra column "isModified" is added that has a value of TRUE for modified fixations, and FALSE otherwise
#
#
# - whereas the monocular version works well, this the binocular version makes a hard-to-justify ad hoc assumption, 
# namely, that very short fixations in one eye get the same treatment in the other eye (i.e., durations are added up etc.)
# this might cause problems: see, e.g., case no. 172
#
# - note: we also need to consider the case of sequences of two or more short fixations
# jochen laubrock, 08/02/08

DCRIT = 30; # maximum difference in ms between duration of the two eyes' fixations (only considered for short fixations)
# if the difference is greater, the fixation is marked (for later deletion of the whole sentence)

relvars <- c("fdur", "sacdur", "x_component", "y_component", "xampl", "yampl", "peakvel")
bL <- aL[,relvars]
bR <- aR[,relvars]

ismodified <- vector(mode = "logical", length = nrow(aL))
bL$ismodified <- ismodified
bR$ismodified <- ismodified
unlikelyfix <- vector(mode = "logical", length = nrow(aL))
bL$unlikelyfix <- unlikelyfix
bR$unlikelyfix <- unlikelyfix

vshrtfix <- (bL$fdur < MIN_DUR) | (bR$fdur < MIN_DUR)
if (sum(vshrtfix)>0){
	vshrtfixIx <- which(vshrtfix)

	# mark unlikely fixations for later removal (outside of this function)
	unlikelyfix <- (abs(aL[, "fdur"] - aR[,  "fdur"]) > DCRIT) & vshrtfix
	unlikelyfixIx <- which(unlikelyfix)

	vshrtfixSeq <- parseq(vshrtfixIx)
	isModified <- vshrtfix

	# we might need to consider this special case. right now we only print a diagnostic message
	if (sum(vshrtfixSeq[,3]>1)) {
		print("removeShortFixations: at least one sequence of two or more extremely short fixations exists")
	}

	# inspect a few cases	# n=57; a[(badfixIx[n]-2):(badfixIx[n]+2),]

	# now remove short fixations by adjusting neighboring fixations
	# case distinction
	# (1) first fixation in sentence: shift firstLastInSentence flag one up, then remove fixation (no adjustment of sacc pars needed)
	isFirstInSentence <- vshrtfixIx %in% which(aL$firstLastInSentence==1 | aR$firstLastInSentence==1)
	if (sum(isFirstInSentence)>0){
		bL[vshrtfixIx[isFirstInSentence]+1,"firstLastInSentence"] <- 1 # set following fix to be the first in sentence
		bR[vshrtfixIx[isFirstInSentence]+1,"firstLastInSentence"] <- 1 # set following fix to be the first in sentence
	}
	
	# (2) brief pseudo-fixation caused by corrective (?) saccade: add saccade duration to statistics of preceding sacc, then remove
	# note: there are some strange cases like id 58, sn 52 (short fix followed by long sacc) which require the argmax treatment, see below
	ix = vshrtfixIx[!isFirstInSentence]
	if (length(ix)>0) {
		idxsL = matrix(nrow=length(ix), ncol=3)
		idxsR = matrix(nrow=length(ix), ncol=3)
		cnt=1
		for (i in ix) {
			idxsL[cnt,1] <- (which.max(abs(bL[(i-1):i, "peakvel"])))
			idxsL[cnt,2] <- (which.max(abs(bL[(i-1):i, "xampl"])))
			idxsL[cnt,3] <- (which.max(abs(bL[(i-1):i, "yampl"])))
			idxsR[cnt,1] <- (which.max(abs(bR[(i-1):i, "peakvel"])))
			idxsR[cnt,2] <- (which.max(abs(bR[(i-1):i, "xampl"])))
			idxsR[cnt,3] <- (which.max(abs(bR[(i-1):i, "yampl"])))
			cnt=cnt+1
		}

		bL[ix-1, "fdur"] 		<- bL[ix, "fdur"] 		 + bL[ix-1, "fdur"] # add duration of fixations
		bL[ix-1, "sacdur"] 		<- bL[ix, "sacdur"] 	 + bL[ix-1, "sacdur"] # add duration of saccades
		bL[ix-1, "x_component"] <- bL[ix, "x_component"] + bL[ix-1, "x_component"] # add up x components of saccades
		bL[ix-1, "y_component"] <- bL[ix, "y_component"] + bL[ix-1, "y_component"] # add up y components of saccades
		bL[ix-1, "peakvel"]   		<- bL[ix + idxsL[,1] - 2, "peakvel"] 		# set to argmax velocity of saccades
		bL[ix-1, "xampl"] 		<- bL[ix + idxsL[,2] - 2, "xampl"] 	# set to argmax xampl of saccades		
		bL[ix-1, "yampl"] 		<- bL[ix + idxsL[,3] - 2, "yampl"] 	# set to argmax yampl of saccades		
		bL[ix-1, "isModified"] <- TRUE

		bR[ix-1, "fdur"] 		<- bR[ix, "fdur"] 		 + bR[ix-1, "fdur"] # add duration of fixations
		bR[ix-1, "sacdur"] 		<- bR[ix, "sacdur"] 	 + bR[ix-1, "sacdur"] # add duration of saccades
		bR[ix-1, "x_component"] <- bR[ix, "x_component"] + bR[ix-1, "x_component"] # add up x components of saccades
		bR[ix-1, "y_component"] <- bR[ix, "y_component"] + bR[ix-1, "y_component"] # add up y components of saccades
		bR[ix-1, "peakvel"] 		<- bR[ix + idxsR[,1] - 2, "peakvel"] 		# set to argmax velocity of saccades
		bR[ix-1, "xampl"] 		<- bR[ix + idxsR[,2] - 2, "xampl"] 	# set to argmax xampl of saccades		
		bR[ix-1, "yampl"] 		<- bR[ix + idxsR[,3] - 2, "yampl"] 	# set to argmax yampl of saccades		
		bR[ix-1, "isModified"] <- TRUE
	} 
	
	if (sum(unlikelyfix)>0) {
		notfirstInSentence <- aL[unlikelyfixIx, "firstLastInSentence"]!=1 & aR[unlikelyfixIx, "firstLastInSentence"]!=1
		bL[unlikelyfixIx[notfirstInSentence]-1,  "unlikelyfix"] <- TRUE
		bR[unlikelyfixIx[notfirstInSentence]-1,  "unlikelyfix"] <- TRUE
	}

	aL[,relvars] <- bL[,relvars]
	aL$isModified <- bL$isModified
	aL$unlikelyfix <- bL$unlikelyfix
	aL <- aL[!vshrtfix,] # remove short fixations

	aR[,relvars] <- bR[,relvars]
	aR$isModified <- bR$isModified
	aR$unlikelyfix <- bR$unlikelyfix
	aR <- aR[!vshrtfix,] # remove short fixations
}	
aamod <- list(aL, aR)	
return(aamod)
}
